require 'dome/helpers/view_model'

module Dome
  class Exhibit < SimpleDelegator
    include Helpers::ViewModel

    TIMESTAMP_ATTRS = %i(
      created_at updated_at deleted_at created_on updated_on deleted_on
    )

    attr_reader :model
    delegate :id, to: :model

    class << self
      def view_path
        "app/exhibits"
      end

      # decorate_methods(*names) { |x, *ys| ... }
      def decorate_methods(*names, &block)
        names.each do |name|
          define_method(name) { |*xs| instance_exec(@model.send(name), *xs, &block) }
        end
      end

      def localize_timestamps(attrs = TIMESTAMP_ATTRS, format: :default)
        decorate_methods(*attrs) { |x| l(x, format: format) }
      end
    end

    def initialize(_model, view_context = nil)
      raise 'Decorationg Nil object not allowed!' unless _model

      @model, @view_context = _model, view_context
      super(_model)
    end

    def class
      model.class
    end

    def to_model
      self
    end

    def attr_name_with_value(attr_name, separator: ': ')
      unless self.class.respond_to?(:human_attribute_name)
        raise "You must implement 'human_attribute_name' method on a class"
      end
      [self.class.human_attribute_name(attr_name), send(attr_name)].join(separator)
    end

    def attr_names_with_values(*attr_names, separator: ': ')
      attr_names.map { |name| attr_name_with_value(name, separator: separator) }
    end
  end
end
