require 'active_support/concern'

require 'dome/helpers/html_drawer'

module Dome::Helpers
  module ViewModel
    extend ActiveSupport::Concern

    included do
      attr_reader :view_context
      alias :h :view_context

      # Added for compatibility with Devise, Rails, etc.
      delegate :request, :translate, :localize, to: :view_context
      alias :t :translate
      alias :l :localize
    end

    module ClassMethods
      def view_path
        "app/views"
      end
    end

    # TODO: if respond_to? :render
    def render(name = nil, vars = {}, opts = {})
      setup_view_path
      @view_context.render(
        name ? { file: name, locals: vars }.merge(opts) : opts
      )
    end

    def compile(args, separator = "\n", do_compact = true, &block)
      separator = h.tag(separator) if separator.is_a?(Symbol)
      (do_compact ? args.compact : args)
        .join(block_given? ? block.() : separator)
        .html_safe
    end

    def draw(&block)
      Dome::Helpers::HtmlDrawer.new(@view_context, &block)
    end

    def draw_with(&block)
      draw { |x| [x, block.call(x)].join("\n").html_safe  }
    end

    private

    # TODO: if respond_to? :controller; if respond_to? :prepend_view_path
    def setup_view_path
      return if @setup_view_path

      c = @view_context.respond_to?(:controller) ? @view_context.controller : @view_context
      @setup_view_path = c.prepend_view_path File.join [
        self.class.view_path,
        self.class.name.underscore
      ]
    end
  end
end
